package main

type API struct {
	BasePath  string                `yaml:"basepath"`
	Host      string                `yaml:"host"`
	Types     map[string]*APIType   `yaml:"types"`
	Methods   map[string]*APIMethod `yaml:"methods"`
	Namespace string
}
type APIFieldTag struct {
	Key  string
	Name string
	Opts []string
}
type APIField struct {
	Type   string `yaml:"type"`
	Array  bool   `yaml:"array"`
	Desc   string `yaml:"desc"`
	Map    bool   `yaml:"map"`
	Mapkey string `yaml:"mapkey"`
	Mapval string `yaml:"mapval"`
	Tags   map[string]APIFieldTag
}

func (a *APIField) String() string {
	if a.Array {
		return "[]" + a.Type
	} else {
		return a.Type
	}
}

type APIType struct {
	Name   string               `yaml:"name"`
	Desc   string               `yaml:"desc"`
	Fields map[string]*APIField `yaml:"fields"`
	Col    string               `yaml:"col"`
}

type APIParamType struct {
	Typename  string
	Ispointer bool
	IsArray   bool
}

type APIMethod struct {
	Desc    string `yaml:"desc"`
	Verb    string `yaml:"verb"`
	Path    string `yaml:"path"`
	Perm    string `yaml:perm`
	ReqType *APIParamType
	ResType *APIParamType
	Raw     bool `yaml:"raw"`
}

func APIParamTypeToString(t *APIParamType) string {
	ret := ""
	if t.IsArray {
		ret = "[]"
		if t.Ispointer {
			ret = ret + "*"
		}
		ret = ret + t.Typename
		return ret
	}
	if t.Ispointer {
		ret = ret + "&"
	}
	ret = ret + t.Typename
	return ret
}

func APIParamTypeDecToString(t *APIParamType) string {
	ret := ""
	if t.IsArray {
		ret = "[]"
		if t.Ispointer {
			ret = ret + "*"
		}
		ret = ret + t.Typename
		return ret
	}
	if t.Ispointer {
		ret = ret + "*"
	}
	ret = ret + t.Typename
	return ret
}

func APIParamTypeUseRef(t *APIParamType) string {
	if t.IsArray || !t.Ispointer {
		return "&"
	}
	return ""
}
